//## begin module%1.4%.codegen_version preserve=yes
//   Read the documentation to learn more about C++ code generator
//   versioning.
//## end module%1.4%.codegen_version

//## begin module%4AB7BA120322.cm preserve=no
//	  %X% %Q% %Z% %W%
//## end module%4AB7BA120322.cm

//## begin module%4AB7BA120322.cp preserve=no
//## end module%4AB7BA120322.cp

//## Module: PlotGroupCreatorClasses%4AB7BA120322; Package body
//## Subsystem: XSPlot::Plot%3D625E94026B
//## Source file: /local/data/xerxes1/gordon/headas/Xspec/src/XSPlot/Plot/PlotGroupCreatorClasses.cxx

//## begin module%4AB7BA120322.additionalIncludes preserve=no
//## end module%4AB7BA120322.additionalIncludes

//## begin module%4AB7BA120322.includes preserve=yes
#include <XSContainer.h>
#include <XSstreams.h>
#include <XSFit/MCMC/Chain.h>
#include <XSFit/MCMC/ChainManager.h>
#include <XSModel/Data/Detector/Response.h>
#include <XSModel/GlobalContainer/DataContainer.h>
#include <XSModel/GlobalContainer/ModelContainer.h>
#include <XSModel/GlobalContainer/ModelTypes.h>
#include <XSModel/Model/Model.h>
#include <XSModel/Model/Component/SumComponent.h>
#include <XSPlot/Plot/PlotGroup.h>
#include <XSUtil/Error/Error.h>
#include <XSUtil/FunctionUtils/FunctionUtility.h>
#include <memory>
#include <sstream>
#include <utility>
//## end module%4AB7BA120322.includes

// PlotGroupCreatorClasses
#include <XSPlot/Plot/PlotGroupCreatorClasses.h>
//## begin module%4AB7BA120322.declarations preserve=no
//## end module%4AB7BA120322.declarations

//## begin module%4AB7BA120322.additionalDeclarations preserve=yes
using namespace XSContainer;
//## end module%4AB7BA120322.additionalDeclarations


// Class CreateModelPlotGroups 

CreateModelPlotGroups::CreateModelPlotGroups()
  //## begin CreateModelPlotGroups::CreateModelPlotGroups%4AB7CEF5026D_const.hasinit preserve=no
  //## end CreateModelPlotGroups::CreateModelPlotGroups%4AB7CEF5026D_const.hasinit
  //## begin CreateModelPlotGroups::CreateModelPlotGroups%4AB7CEF5026D_const.initialization preserve=yes
  :PlotGroupCreator(),
   m_modName()
  //## end CreateModelPlotGroups::CreateModelPlotGroups%4AB7CEF5026D_const.initialization
{
  //## begin CreateModelPlotGroups::CreateModelPlotGroups%4AB7CEF5026D_const.body preserve=yes
  //## end CreateModelPlotGroups::CreateModelPlotGroups%4AB7CEF5026D_const.body
}


CreateModelPlotGroups::~CreateModelPlotGroups()
{
  //## begin CreateModelPlotGroups::~CreateModelPlotGroups%4AB7CEF5026D_dest.body preserve=yes
  //## end CreateModelPlotGroups::~CreateModelPlotGroups%4AB7CEF5026D_dest.body
}



//## Other Operations (implementation)
std::vector<PlotGroup*> CreateModelPlotGroups::createPlotGroups (const PlotSettings& settings)
{
  //## begin CreateModelPlotGroups::createPlotGroups%4AB7CFBB0297.body preserve=yes
   
   string mapName(m_modName);
   if ( !mapName.size() ) mapName = Model::DEFAULT();

   if (models->modelSet().empty())
   {
      throw YellowAlert(" no model defined\n");
   }
   std::pair<ModelMapConstIter, ModelMapConstIter> itRange = models->modelSet().equal_range(mapName);

   if (itRange.first == itRange.second)
   {
      string msg(" model not found\n");
      if (mapName == Model::DEFAULT())
      {
         msg += "No model has been entered with the default name\n";
      }
      else
      {
         msg += "No model has been entered with name = " + mapName + "\n";
      }
      throw YellowAlert(msg);
   }  

   if (datasets->numSourcesForSpectra() && itRange.first->second->sourceNumber() > datasets->numSourcesForSpectra())
   {
      string msg("Cannot plot model of higher source number than data allows.\n");
      throw YellowAlert(msg);
   }

   std::vector<PlotGroup*> plotGroups;
   ModelMapConstIter itMod (itRange.first);
   
   try
   {
      while (itMod  != itRange.second )
      {
         const Model* mod = itMod->second;
         const ArrayContainer& energies = mod->energy();
         ArrayContainer::const_iterator itEn = energies.begin();
         ArrayContainer::const_iterator itEnEnd = energies.end();
         while (itEn != itEnEnd)
         {
            int specNum = itEn->first;
            const RealArray& eng = itEn->second;
            PlotGroup* gr = initializePlotGroup(mod, specNum, eng, settings);
            plotGroups.push_back(gr);
            ++itEn;
         }
         ++itMod;
      }
   }
   catch (...)
   {
      for (size_t i=0; i<plotGroups.size(); ++i)
         delete plotGroups[i];
      throw;
   }
   
   return plotGroups;
  //## end CreateModelPlotGroups::createPlotGroups%4AB7CFBB0297.body
}

PlotGroup* CreateModelPlotGroups::initializePlotGroup (const Model* model, int specNum, const RealArray& energy, const PlotSettings& settings)
{
  //## begin CreateModelPlotGroups::initializePlotGroup%4AC37F220091.body preserve=yes
  const int NE (energy.size() - 1); 
   
  std::auto_ptr<PlotGroup> apGr(new PlotGroup(NE,1,false)); 
  PlotGroup* gr = apGr.get();
  
  // set the x axis array.

  for (int j = 0; j < NE; ++j)
  {
     // dummies are only used for channel mode, which has to be
     // switched off.
     int dummy(0);
     const Real chanEmin = energy[j]*(1.0 + settings.redshiftToSource());
     const Real chanEmax = energy[j+1]*(1.0 + settings.redshiftToSource());
     setXaxis(settings,gr,dummy,dummy,chanEmin,chanEmax,j);      
  }      

  const RealArray& flux = model->modelFlux(specNum);
  const RealArray& fluxErr = model->modelFluxError(specNum);

  gr->model[0].data.resize(NE,0);
  for ( int j = 0; j < NE; ++j)
  {
     gr->model[0].data[j] = flux[j];
  }
  if ( fluxErr.size())
  {
     gr->model[0].errors.push_back(std::vector<Real>(NE));
     for ( int j = 0; j < NE; ++j)
     {
        gr->model[0].errors[0][j] = fluxErr[j];
     }     
  }
  else 
  {
     gr->model[0].errors.push_back(std::vector<Real>());             
  }

  // if sources are present and required, put them in the right 
  // place too.

  const SourceList& sources = model->sources();
  size_t numberOfSourcesInModel(sources.size()); 
  if (numberOfSourcesInModel > 1)
  {
     const size_t& n = gr->n;
     PlotVectorList& currentSources = gr->sources[0];

     SourceList::const_iterator sl = sources.begin();    

     for (size_t j = 0; j < numberOfSourcesInModel; ++j)
     {
         currentSources.push_back(PlotVector(n));
         PlotVector& cs = currentSources.back();
         const SumComponent* c = *sl;
         for (size_t k = 0; k < n; ++k)
         {
            cs.data[k]  = c->photonArray(specNum)[k];       
         }
         ++sl;
     }
  }
                
  return apGr.release();
  //## end CreateModelPlotGroups::initializePlotGroup%4AC37F220091.body
}

// Additional Declarations
  //## begin CreateModelPlotGroups%4AB7CEF5026D.declarations preserve=yes
  //## end CreateModelPlotGroups%4AB7CEF5026D.declarations

// Class CreateDemPlotGroups 

CreateDemPlotGroups::CreateDemPlotGroups()
  //## begin CreateDemPlotGroups::CreateDemPlotGroups%4AB7D061021C_const.hasinit preserve=no
  //## end CreateDemPlotGroups::CreateDemPlotGroups%4AB7D061021C_const.hasinit
  //## begin CreateDemPlotGroups::CreateDemPlotGroups%4AB7D061021C_const.initialization preserve=yes
  : PlotGroupCreator()
  //## end CreateDemPlotGroups::CreateDemPlotGroups%4AB7D061021C_const.initialization
{
  //## begin CreateDemPlotGroups::CreateDemPlotGroups%4AB7D061021C_const.body preserve=yes
  //## end CreateDemPlotGroups::CreateDemPlotGroups%4AB7D061021C_const.body
}


CreateDemPlotGroups::~CreateDemPlotGroups()
{
  //## begin CreateDemPlotGroups::~CreateDemPlotGroups%4AB7D061021C_dest.body preserve=yes
  //## end CreateDemPlotGroups::~CreateDemPlotGroups%4AB7D061021C_dest.body
}



//## Other Operations (implementation)
std::vector<PlotGroup*> CreateDemPlotGroups::createPlotGroups (const PlotSettings& settings)
{
  //## begin CreateDemPlotGroups::createPlotGroups%4AC38929013E.body preserve=yes
   int nPts = static_cast<int>(FunctionUtility::DEM().size());

   //did demSize come back with a size of 0? Could indicate model 
   //wasn't calculated properly, or at all. Force a calculation
   if(!nPts)
   {
       ModelMap::const_iterator 
	   itModSet = models->modelSet().begin(),
	   itModSetEnd = models->modelSet().end();

       while(itModSet != itModSetEnd)
       {
	   models->calculate(itModSet->first);
	   ++itModSet;
       }
       nPts = static_cast<int>(FunctionUtility::DEM().size());

       //if it's still 0, them we have an unrecoverable error, quit
       if(!nPts)
	   throw YellowAlert("Unable to do dem plot: dem size is 0\n");
   }

   PlotGroup* newGroup = new PlotGroup(nPts);

   for(int i = 0; i < nPts; ++i) 
   {
       newGroup->xAxis.data[i] = FunctionUtility::tempsDEM()[i];
       newGroup->yData.data[i] = FunctionUtility::DEM()[i];
   }
   newGroup->xAxis.errors.resize(0);
   newGroup->yData.errors.resize(0);

   std::vector<PlotGroup*> plotGroups(1,newGroup);
   
   return plotGroups;
  //## end CreateDemPlotGroups::createPlotGroups%4AC38929013E.body
}

// Additional Declarations
  //## begin CreateDemPlotGroups%4AB7D061021C.declarations preserve=yes
  //## end CreateDemPlotGroups%4AB7D061021C.declarations

// Class CreateEfficiencyPlotGroups 

CreateEfficiencyPlotGroups::CreateEfficiencyPlotGroups()
  //## begin CreateEfficiencyPlotGroups::CreateEfficiencyPlotGroups%4AB7D19801AD_const.hasinit preserve=no
  //## end CreateEfficiencyPlotGroups::CreateEfficiencyPlotGroups%4AB7D19801AD_const.hasinit
  //## begin CreateEfficiencyPlotGroups::CreateEfficiencyPlotGroups%4AB7D19801AD_const.initialization preserve=yes
  : PlotGroupCreator()
  //## end CreateEfficiencyPlotGroups::CreateEfficiencyPlotGroups%4AB7D19801AD_const.initialization
{
  //## begin CreateEfficiencyPlotGroups::CreateEfficiencyPlotGroups%4AB7D19801AD_const.body preserve=yes
  //## end CreateEfficiencyPlotGroups::CreateEfficiencyPlotGroups%4AB7D19801AD_const.body
}


CreateEfficiencyPlotGroups::~CreateEfficiencyPlotGroups()
{
  //## begin CreateEfficiencyPlotGroups::~CreateEfficiencyPlotGroups%4AB7D19801AD_dest.body preserve=yes
  //## end CreateEfficiencyPlotGroups::~CreateEfficiencyPlotGroups%4AB7D19801AD_dest.body
}



//## Other Operations (implementation)
std::vector<PlotGroup*> CreateEfficiencyPlotGroups::createPlotGroups (const PlotSettings& settings)
{
  //## begin CreateEfficiencyPlotGroups::createPlotGroups%4AC3894701CF.body preserve=yes
   std::vector<PlotGroup*> plotGroups;
   const size_t nSpec = datasets->numberOfSpectra();
   
   try
   {
      for (size_t iSpec=1; iSpec<=nSpec; ++iSpec)
      {
         PlotGroup* gr=0;
         SpectralData* spectrum = datasets->lookup(iSpec);
         const std::vector<Response*>& dets = spectrum->detector();
         bool anyDets = false;
         for (size_t iDet=0; iDet<dets.size(); ++iDet)
         {
            const Response* resp = dets[iDet];
            if (resp)
            {
               anyDets = true;
               const RealArray& energy = resp->energies();
               const size_t NE (energy.size() - 1);
               gr = new PlotGroup(NE);
               plotGroups.push_back(gr);
               
               for (size_t j = 0; j < NE; ++j)
               {
                     // dummies are only used for channel mode, which has to be
                     // switched off.
                     int dummy(0);
                     setXaxis(settings,gr,dummy,dummy,energy[j],energy[j+1],j);      
               }                     
            }
         } // end dets loop

         if (!anyDets)
         {
               // One could put in a dummy response here, but for the uses
               // this function is likely to be put to the plot won't be very
               // interesting (e.g. detector efficiency).  Still continue
               // loop though, don't throw.
               std::ostringstream ss;
               ss << "\n***XSPEC Error: No energies defined for spectrum " 
                  << iSpec << "\n";
               tcerr << ss.str() << std::endl;
         }

      } // end spectra loop
   }
   catch (...)
   {
      for (size_t i=0; i<plotGroups.size(); ++i)
         delete plotGroups[i];
      throw;
   }
   
   if (plotGroups.empty())
     throw YellowAlert("No response energies for any loaded spectra.\n");
   
   return plotGroups;
  //## end CreateEfficiencyPlotGroups::createPlotGroups%4AC3894701CF.body
}

// Additional Declarations
  //## begin CreateEfficiencyPlotGroups%4AB7D19801AD.declarations preserve=yes
  //## end CreateEfficiencyPlotGroups%4AB7D19801AD.declarations

// Class CreateChainPlotGroups 

CreateChainPlotGroups::CreateChainPlotGroups()
  //## begin CreateChainPlotGroups::CreateChainPlotGroups%4AB7D2610026_const.hasinit preserve=no
  //## end CreateChainPlotGroups::CreateChainPlotGroups%4AB7D2610026_const.hasinit
  //## begin CreateChainPlotGroups::CreateChainPlotGroups%4AB7D2610026_const.initialization preserve=yes
  : PlotGroupCreator(),
   m_xColNum(0),
   m_yColNum(0)
  //## end CreateChainPlotGroups::CreateChainPlotGroups%4AB7D2610026_const.initialization
{
  //## begin CreateChainPlotGroups::CreateChainPlotGroups%4AB7D2610026_const.body preserve=yes
  //## end CreateChainPlotGroups::CreateChainPlotGroups%4AB7D2610026_const.body
}


CreateChainPlotGroups::~CreateChainPlotGroups()
{
  //## begin CreateChainPlotGroups::~CreateChainPlotGroups%4AB7D2610026_dest.body preserve=yes
  //## end CreateChainPlotGroups::~CreateChainPlotGroups%4AB7D2610026_dest.body
}



//## Other Operations (implementation)
std::vector<PlotGroup*> CreateChainPlotGroups::createPlotGroups (const PlotSettings& settings)
{
  //## begin CreateChainPlotGroups::createPlotGroups%4AC3890B0259.body preserve=yes
   size_t totalChainLength = 0;
   ChainManager* chainManager = ChainManager::Instance();

   //more efficient to get the total number of chains and allocate 
   //memory once in PlotGroup rather than growing it as necessary
   ChainManager::ChainContainer::const_iterator
       itChain = chainManager->chains().begin(),
       itChainEnd = chainManager->chains().end();

   while(itChain != itChainEnd)
   {
       totalChainLength += itChain->second->length();
       ++itChain;
   }
   std::auto_ptr<PlotGroup> apGr(new PlotGroup(totalChainLength));
   PlotGroup* chainGroup = apGr.get();
   chainGroup->xAxis.errors.clear();
   chainGroup->yData.errors.clear();
   
   //read points from chain file and fill new PlotGroup	
   std::vector<Real> row;
   size_t idx=0;
   itChain = chainManager->chains().begin();
   while (itChain != itChainEnd)
   {
      const Chain& chain = *(itChain->second);
      chain.openForReadPoints();
      size_t chainLength = chain.length();

      if (m_xColNum > 0)
      {
	 for(size_t i = 0; i < chainLength; ++i, ++idx)
	 {
	     chain.readPoint(row);

	     //user-supplied column nums are 1-based
	     chainGroup->xAxis.data[idx] = row[m_xColNum - 1];
	     chainGroup->yData.data[idx] = row[m_yColNum - 1];
	 }
      }
      else
      {
	 for(size_t i = 0; i < chainLength; ++i, ++idx)
	 {
	     chain.readPoint(row);
	     chainGroup->xAxis.data[idx] = idx+1;
	     chainGroup->yData.data[idx] = row[m_yColNum - 1];
	 }
      }

      chain.closeFile();
      ++itChain;
   }
   std::vector<PlotGroup*> plotGroups;
   plotGroups.push_back(apGr.release());   
   return plotGroups;
  //## end CreateChainPlotGroups::createPlotGroups%4AC3890B0259.body
}

// Additional Declarations
  //## begin CreateChainPlotGroups%4AB7D2610026.declarations preserve=yes
  //## end CreateChainPlotGroups%4AB7D2610026.declarations

//## begin module%4AB7BA120322.epilog preserve=yes
//## end module%4AB7BA120322.epilog
